@@ -11,16 +11,16 @@
  * Control glibc, especially to get 64 bit file offsets
  */
 #ifdef __linux
-/* 
+/*
  * Don't use this unless I have to:
  * #define _GNU_SOURCE
  */
-/* 
- * Let me see if I get that straight: 
+/*
+ * Let me see if I get that straight:
  * -first enable "some" additional calls
  */
 #define _LARGEFILE_SOURCE
-/* 
+/*
  * -then enable the 64 bits variants of traditional calls, on seperate names
  */
 #define _LARGEFILE64_SOURCE
@@ -134,6 +134,7 @@ struct options {
   int c;
   int w;
   int S;
+  int T;
 };
 
 struct progstate {
@@ -153,6 +154,11 @@ struct progstate {
 	      * the stream to.  Currently that will always be 1 (stdout) */
 };
 
+static void sigtimer(const int signal)
+{
+  signal_report = 1;
+}
+
 static void sigshutdown(const int signal)
 {
   stopit = 1;
@@ -167,7 +173,7 @@ static void sigchld(const int sig)
 
 
   fprintf(stderr, "sigchld\n");
-  /* 
+  /*
    * In theory we shouldn't have to use nohang, but Linux seems to send
    * SIGCHLD when nothing is available.
    */
@@ -211,7 +217,7 @@ static void sighup(int signal)
 
 static void handle_sighup(void)
 {
-  fprintf(stderr, 
+  fprintf(stderr,
 	  "Received SIGHUP. "
 	  "If this is an error, send another one within 5 seconds\n");
   n_sighups = 1;
@@ -380,7 +386,7 @@ static int open_tcp(const struct options *const o, int mode)
     close(fd);
     fd = newfd;
   } else { /* Connect */
-      
+
     hostname = strdup(spec);
     hostname[port - spec - 1] = '\0';
 
@@ -655,13 +661,14 @@ static void print_kmg(const char *const pre
 }
 
 static void report(const struct options *const o,
-		   const struct progstate *const state, 
+		   const struct progstate *const state,
 		   int curbytes)
 {
   struct timeval t2;
   double sofar;
   double rate;
-    
+  struct itimerval itv;
+
   if (gettimeofday(&t2, NULL) == -1) {
     perror("gettimeofday() failed");
     exit(2);
@@ -680,7 +687,7 @@ static void report(const struct options *const o,
     fprintf(stderr," %.1f s", sofar);
     if (sofar >= 3600.0)
       fprintf(stderr, " (%d:%02d h)", (int)sofar / 3600, (int)sofar % 3600 / 60);
-    else 
+    else
       fprintf(stderr, " (%d:%02d min)", (int)sofar / 60, (int)sofar % 60);
   }
   fprintf(stderr," %.0f B/s", rate);
@@ -692,7 +699,7 @@ static void report(const struct options *const o,
   fprintf(stderr, "\n");
 
   if (o->v >= 2 && (sofar - state->time_firsttransfer) > 0.0) {
-    rate = 
+    rate =
       (
        (double)state->bytes_transferred - (double)state->bytes_firsttransfer
        ) / (sofar - state->time_firsttransfer);
@@ -700,6 +707,14 @@ static void report(const struct options *const o,
     print_kmg(" ", "%.2f", rate, "B/s", stderr);
     fprintf(stderr, "\n");
   }
+
+  if (o->T) {
+    itv.it_interval.tv_sec = o->T;
+    itv.it_interval.tv_usec = 0;
+    itv.it_value.tv_sec = o->T;
+    itv.it_value.tv_usec = 0;
+    setitimer(ITIMER_REAL, &itv, NULL);
+  }
 }
 
 #if HAVE_POLL_H
@@ -740,8 +755,8 @@ static void pollloop(const struct options *const o
 	) {
       want_to_read = 1;
     } else {
-      /* 
-       * No space for another read(). 
+      /*
+       * No space for another read().
        * Try to reset buffer pointers if everything has been written.
        * In that case, we also want to read into the new buffer.
        * Otherwise, we want to write, but not read.
@@ -841,11 +856,11 @@ static void pollloop(const struct options *const o
 static void loop(struct options *const o, struct progstate *const state)
 {
   char *buf; /* Holds the whole buffer o->B*/
-  char *curbuf; /* Iterates over buf when reading and leave after last 
+  char *curbuf; /* Iterates over buf when reading and leave after last
 		 * read position */
   char *curbuf2; /* Iterate over buf when writing */
   int nbytes = 0; /* Just one system call read() */
-  long long bytes_read = 0; /* Overall, both loops */
+  int bytes_read = 0; /* Overall, both loops */
   int pipefd[2];
 
   state->pid = getpid();
@@ -910,13 +925,13 @@ static void loop(struct options *const o, struct progstate *const state)
 
 	for (i = 0; i < o->B; i++) {
 	  /* Newlines below 80 chars and at end of buffer */
-	  if ((i+1) % 76 == 0 || i == o->n - 1) 
+	  if ((i+1) % 76 == 0 || i == o->n - 1)
 	    buf[i] = '\n';
 	  else
 	    buf[i] = 'A' + rand() % 26;
 	}
       } else {
-	/* 
+	/*
 	 * Audio mode, generate 440 Hz wave
 	 *
 	 * Do it verbose to do it right
@@ -954,7 +969,7 @@ static void loop(struct options *const o, struct progstate *const state)
   while (stopit == 0) {
     /* Outer loop - encapsulates pairs of read/write loops */
 
-    for (curbuf = buf; stopit == 0 && curbuf + o->b <= buf + o->B; 
+    for (curbuf = buf; stopit == 0 && curbuf + o->b <= buf + o->B;
 	 curbuf += nbytes, bytes_read += nbytes) {
       /* Inner loop, fill buffer with reads() */
 
@@ -968,7 +983,7 @@ static void loop(struct options *const o, struct progstate *const state)
 	double sofar;
 	double theory;
 	double time_to_sleep;
-	
+
 	if (gettimeofday(&t2, NULL) == -1) {
 	  perror("gettimeofday() failed");
 	  exit(2);
@@ -1002,7 +1017,7 @@ static void loop(struct options *const o, struct progstate *const state)
 		    ", should be %g, sleeping %g\n"
 		    , sofar, o->b, theory, time_to_sleep);
 
-	  state->lasttime = ((double)t2.tv_sec + 
+	  state->lasttime = ((double)t2.tv_sec +
 			     (double)t2.tv_usec / 1000000.0);
 
 	  if (sofar < theory) {
@@ -1132,11 +1147,11 @@ static void loop(struct options *const o, struct progstate *const state)
 	  perror("gettimeofday() failed");
 	  exit(2);
 	}
-	state->time_firsttransfer = 
-	  ((double)t.tv_sec + (double)t.tv_usec / 1000000.0) - 
+	state->time_firsttransfer =
+	  ((double)t.tv_sec + (double)t.tv_usec / 1000000.0) -
 	  state->starttime;
       }
-	
+
       if (o->n && state->bytes_transferred >= o->n) {
 	if (o->v > 1)
 	  fprintf(stderr, "#%d finishing on condition 5\n",
@@ -1219,6 +1234,7 @@ static void usage(void)
 	  "-l       include line count in statistics\n"
 	  "-w <n> = Set write block size (-c 5 only)\n"
 	  "-S       Don't output statistic on SIGINFO\n"
+	  "-T <n> = Report throughput every <n> seconds\n"
 	  "SIGINFO causes statistics to be written to stderr\n"
 	  "SIGUSR1 causes statistics to be written to stderr\n"
 	  "SIGUSR2 causes loop end after next buffer transfer\n"
@@ -1252,7 +1268,7 @@ int main(int argc, char *const argv[])
 
   default_options(&o);
 
-  while ((ch = getopt(argc, argv, "b:B:c:i:I:n:o:O:p:St:v:Vl")) != -1) {
+  while ((ch = getopt(argc, argv, "b:B:c:i:I:n:o:O:p:St:T:v:Vl")) != -1) {
     switch(ch) {
     case 'v': {
       if (optarg[1] != '\0' ||
@@ -1265,7 +1281,7 @@ int main(int argc, char *const argv[])
 		"between 0 and 4.\nType -? for usage instructions\n");
 	exit(1);
       }
-      o.v = atoi(optarg); 
+      o.v = atoi(optarg);
       break;
     }
     case 'b': o.b = atoi_kmg(optarg); break;
@@ -1282,6 +1298,7 @@ int main(int argc, char *const argv[])
     case 'p': o.p = strdup(optarg); break;
     case 'V': print_version(); exit(0);
     case 'w': o.w = atoi_kmg(optarg); break;
+    case 'T': o.T = atoi(optarg); break;
     default: usage();
     }
   }
@@ -1291,9 +1308,9 @@ int main(int argc, char *const argv[])
 
   switch (argc) {
   case 0: break;
-  case 1: 
-    if (o.i == NULL) 
-      o.i = strdup(argv[0]); 
+  case 1:
+    if (o.i == NULL)
+      o.i = strdup(argv[0]);
     else
       usage();
     break;
@@ -1302,6 +1319,17 @@ int main(int argc, char *const argv[])
 
   init(&o, &state);
 
+  if (o.T) {
+    struct itimerval itv;
+
+    signal(SIGALRM, sigtimer);
+    itv.it_interval.tv_sec = o.T;
+    itv.it_interval.tv_usec = 0;
+    itv.it_value.tv_sec = o.T;
+    itv.it_value.tv_usec = 0;
+    setitimer(ITIMER_REAL, &itv, NULL);
+  }
+
   if (o.c == 4) {
 #if HAVE_POLL_H
     pollloop(&o, &state);
